    // Copyright (c) 2008 Pedro Quaresma, Coimbra, Portugal
    // Versão 2.0 
    // versão 1.0, desenvolvida por Augusto Pinho
    //   (sob supervisão de Pedro Quaresma)


%{
#include <ctype.h>

  /*
    os índices do vector vão coincidir com os valores da codificação
    dos caracteres, que no caso dos caracteres acentuados é ajustada
    de forma a não haver conflitos, 122+1+32 = 155, 

    122 - valor numérico de "z" (o maior positivo)
    +1  - a próxima posição
    +32 - o valor (mais negativo) de "à"

    isto dado não se poderem considerar índices negativos. O gasto 
    de memória que esta solução implica não é (muito) significativo.

    Uma solução mais genérica tem de passar por adoptar os novos tipos
    "wide chars".


    As maiúsculas vão ser convertidas em minúsculas.
  */

#define MAXCOD 155

int paraminuscula(char c) {
  /* 
     A diferença entre letras maiúsculas e minúsculas é de
     32, no caso das letras não acentuadas pode-se usar a função 
     pré-definida tolower, no caso das letras acentuadas essa função
     não faz nada
  */
  if (c >= -64 && c <= -36) // entre "À" e "Ü"
    return(c+32);
  else
    return(c);
}

%}

   int i;
   double conta_palavra=0;
   int conta_letra=0;     
   int vec[MAXCOD];
   char ch;
   double frequenciaRelativa;
   double d1;
   double d2;

letras [a-zA-Z]|"á"|"à"|"ã"|"â"|"ç"|"é"|"è"|"ê"|"í"|"ì"|"ó"|"ò"|"õ"|"ô"|"ú"|"ù"|"ü"|"Á"|"À"|"Ã"|"Â"|"Ç"|"É"|"È"|"Ê"|"Í"|"Ì"|"Ó"|"Ò"|"Õ"|"Ô"|"Ú"|"Ù"|"Ü"


%% 
{letras}  { 
	   if(yytext[0]<0) {
	     i =-1*paraminuscula(yytext[0])+123;
	     vec[i]++;
	     conta_letra++;
	   }   
	   else {
	     i = tolower(yytext[0]);
	     vec[i]++;
	     conta_letra++;
	   }
          } 

.|\n

%%

main(int argc, char *argv[]) {
  
  // interface linha de comando
  // media <nome_de_ficheiro_entrada> <nome_de_ficheiro_saida>
  
  char *ficheiroEntrada,*ficheiroSaida;
  double total_letras=0;

  FILE *fsai, *fopen();
  
  if (argc<=2) { // valores de entrada insuficientes (2+1)
    printf("\nUtilização: frequencias nome_ficheiro_entrada nome_ficheiro_saída\n\n");
    return;
  }
  // obtêm os argumentos da linha de comando
  ficheiroEntrada=argv[1];
  ficheiroSaida=argv[2];
  printf ("\nFicheiro a Processar: %s\nFicheiro com resultados: %s\n\n",ficheiroEntrada,ficheiroSaida);
  
  if ((yyin = fopen(ficheiroEntrada,"r")) == NULL) {
    printf("Não foi possível abrir o ficheiro %s\n",ficheiroEntrada);
  } 
  else if ((fsai = fopen(ficheiroSaida,"w")) == NULL) {
    printf("Não foi possível abrir para escrita o ficheiro %s\n",ficheiroSaida);
  } 
  else {
    // processar
    yylex();
    // Contar o número total de letras
    for (i=0; i < MAXCOD; i++)
      if(vec[i] != 0) {
	total_letras = total_letras+vec[i];
      }
    
    for (i=0; i < MAXCOD; i++)
      if(vec[i] != 0) {
	if (i>124) {
	  ch=-1*(i-123);   
	  d1=vec[i];
	  frequenciaRelativa=(d1/total_letras)*100; 
	  fprintf(fsai,"%c\t%0.6f\n",ch,frequenciaRelativa);
	}
	else {
	  ch=i;
	  d1=vec[i];
	  frequenciaRelativa=(d1/total_letras)*100;
	  fprintf(fsai,"%c\t%0.6f\n",ch,frequenciaRelativa);
	}
      }
    printf("\n%0.0f\n",total_letras);
  }
}
